@@ -1,19 +1,30 @@
-__version__ = "0.1.0"
+__version__ = "1.1.1"
 
-import time
 from multiprocessing import Process, Queue
 from threading import Timer
-from typing import Any, Callable, Dict, Generic, Iterable, List, Optional, TypeVar
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generic,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    TypeVar,
+    cast,
+)
+
 
 from olympipe.pipes.reduce import ReducePipe
-from .pipes.generic import GenericPipe
-from .pipes.task import TaskPipe
-from .pipes.filter import FilterPipe
-from .pipes.explode import ExplodePipe
-from .pipes.batch import BatchPipe
-from .pipes.instance import ClassInstancePipe
-from .pipes.timebatch import TimeBatchPipe
-from .dispatcher import Dispatcher
+from olympipe.pipes.generic import GenericPipe
+from olympipe.pipes.task import TaskPipe
+from olympipe.pipes.filter import FilterPipe
+from olympipe.pipes.explode import ExplodePipe
+from olympipe.pipes.batch import BatchPipe
+from olympipe.pipes.instance import ClassInstancePipe
+from olympipe.pipes.timebatch import TimeBatchPipe
+from olympipe.dispatcher import Dispatcher
 
 R = TypeVar("R")
 S = TypeVar("S")
@@ -31,40 +42,46 @@ class Pipeline(Generic[R]):
     def get_new_queue() -> "Queue[R]":
         return Queue(maxsize=Pipeline.max_queue_size)
 
-    def __init__(self, datas: Optional[Iterable[R]] = None):
+    def __init__(
+        self,
+        datas: Optional[Iterable[R]] = None,
+        links: Optional[List["Tuple[Pipeline[Any], Process, Queue[Any]]"]] = None,
+    ):
         self._source: "Queue[R]" = Pipeline.get_new_queue()
         self._parents: List["Process"] = []
         self.dispatcher = Dispatcher(self._source)
         self._register(self.dispatcher)
-        if datas is not None:
-            Timer(0.05, self.source, (datas,)).start()
+        self._links: List["Tuple[Pipeline[Any], Process, Queue[Any]]"] = (
+            [] if links is None else links
+        )
+        self._datas = datas
 
-    def source(self, datas: Iterable[R]) -> None:
-        for data in datas:
-            self._source.put(data)
+    def start(self):
+        input_pipe = self._links[0][0] if len(self._links) > 0 else self
+        if input_pipe._datas is not None:
+            for data in input_pipe._datas:
+                input_pipe._source.put(data)
+        input_pipe._source.put(GenericPipe.get_kill_word())
 
-        self.auto_close()
-
-    def auto_close(self):
-        while not self._source.empty():
-            time.sleep(0.15)
-        self._source.put(GenericPipe.get_kill_word())
-
-    def _register(self, process: Process):
+    def _register(
+        self, process: Process, output_pipe: Optional["Pipeline[Any]"] = None
+    ):
         """_register
         This method will register a process as parent
         to know when it should stop
         """
+        if output_pipe is not None:
+            output_pipe._links = [*self._links, (self, process, output_pipe._source)]
         self._parents.append(process)
 
     def task(self, task: Callable[[R], S], count: int = 1) -> "Pipeline[S]":
         assert count >= 1
         target_queue: "Queue[R]" = Pipeline.get_new_queue()
-        output_pipe: "Pipeline[S]" = Pipeline()
+        output_pipe: "Pipeline[S]" = Pipeline(links=self._links)
         self.dispatcher.dispatch_to([target_queue], count)
         for _ in range(count):
             p = TaskPipe(target_queue, task, output_pipe._source)
-            self._register(p)
+            self._register(p, output_pipe)
         return output_pipe
 
     def class_task(
@@ -90,7 +107,7 @@ class Pipeline(Generic[R]):
                 class_args,
                 class_kwargs,
             )
-            self._register(p)
+            self._register(p, output_pipe)
         return output_pipe
 
     def explode(self, explode_function: Callable[[R], Iterable[S]]) -> "Pipeline[S]":
@@ -98,7 +115,7 @@ class Pipeline(Generic[R]):
         output_pipe: "Pipeline[S]" = Pipeline()
         p = ExplodePipe(target_queue, explode_function, output_pipe._source)
 
-        self._register(p)
+        self._register(p, output_pipe)
         self.dispatcher.dispatch_to([target_queue])
         return output_pipe
 
@@ -108,7 +125,7 @@ class Pipeline(Generic[R]):
         p = BatchPipe(target_queue, output_pipe._source, count)
         self.dispatcher.dispatch_to([target_queue])
 
-        self._register(p)
+        self._register(p, output_pipe)
         return output_pipe
 
     def temporal_batch(self, time_interval: float) -> "Pipeline[Iterable[R]]":
@@ -117,7 +134,7 @@ class Pipeline(Generic[R]):
         p = TimeBatchPipe(target_queue, output_pipe._source, time_interval)
         self.dispatcher.dispatch_to([target_queue])
 
-        self._register(p)
+        self._register(p, output_pipe)
         return output_pipe
 
     def filter(self, filter_function: Callable[[R], bool]) -> "Pipeline[R]":
@@ -126,12 +143,12 @@ class Pipeline(Generic[R]):
         p = FilterPipe(target_queue, filter_function, output_pipe._source)
         self.dispatcher.dispatch_to([target_queue])
 
-        self._register(p)
+        self._register(p, output_pipe)
         return output_pipe
 
     @staticmethod
     def print_return(data: S) -> S:
-        print(f"debug_{data}")
+        print(f"debug_pipe {data}")
         return data
 
     def debug(self) -> "Pipeline[R]":
@@ -150,20 +167,61 @@ class Pipeline(Generic[R]):
         output_pipe: "Pipeline[T]" = Pipeline()
 
         p = ReducePipe(target_queue, output_pipe._source, accumulator, reducer)
-        self._register(p)
+        self._register(p, output_pipe)
         self.dispatcher.dispatch_to([target_queue])
         return output_pipe
 
-    def wait_and_reduce(self, accumulator: T, reducer: Callable[[R, T], T]) -> "T":
+    def build_dot_graph(self, debug_file: str):
+        links: List[Tuple[str, str, str, str]] = []
+        for t, u, v in self._links:
+            tn = ""
+            try:
+                tn = cast(Any, u)._task.__name__
+                tn = "Debug" if tn == "print_return" else tn
+            except:
+                pass
+            frm = t._source.__repr__().split(" ")[-1][:-1]
+            to = v.__repr__().split(" ")[-1][:-1]
+            links.append((frm, to, u.__class__.__name__[:-4], tn))
+
+        renamer: Dict[str, str] = {}
+        count = 0
+        for i, link in enumerate(links):
+            frm, to, name, tn = link
+            if frm not in renamer:
+                renamer[frm] = str(count)
+                count += 1
+            frm = renamer[frm]
+            if to not in renamer:
+                renamer[to] = str(count)
+                count += 1
+            to = renamer[to]
+            links[i] = (frm, to, name, tn)
+
+        with open(debug_file, "w") as f:
+            _ = f.write("digraph G {\n")
+            for frm, to, name, tn in links:
+                _ = f.write(f'"{frm}" -> "{to}" [label="{name} {tn}"]\n')
+            _ = f.write("}\n")
+
+    def wait_and_reduce(
+        self,
+        accumulator: T,
+        reducer: Callable[[R, T], T],
+        debug_graph: Optional[str] = None,
+    ) -> "T":
         o = self.reduce(accumulator, reducer)
-        [[res]] = Pipeline.wait_for_all_results([o])
+        if debug_graph is not None:
+            o.build_dot_graph(debug_graph)
+        [[res]] = Pipeline._wait_for_all_results([o])
         return res
 
     @staticmethod
-    def wait_for_all_completions(
+    def _wait_for_all_completions(
         pipes: List["Pipeline[Any]"], timeout: Optional[int] = None
     ) -> None:
         output_queues: List[Queue[Any]] = [p._prepare_output_buffer() for p in pipes]
+        Timer(0.015, pipes[0].start).start()
 
         for o in output_queues:
             while True:
@@ -172,10 +230,11 @@ class Pipeline(Generic[R]):
                     break
 
     @staticmethod
-    def wait_for_all_results(pipes: List["Pipeline[Any]"]) -> List[List[Any]]:
+    def _wait_for_all_results(pipes: List["Pipeline[Any]"]) -> List[List[Any]]:
         output_queues: List[Optional[Queue[Any]]] = [
             p._prepare_output_buffer() for p in pipes
         ]
+        Timer(0.015, pipes[0].start).start()
 
         outputs: List[List[Any]] = [[] for _ in pipes]
 
@@ -190,19 +249,52 @@ class Pipeline(Generic[R]):
                         output_queues[i] = None
                     else:
                         outputs[i].append(packet)
-                except:
+                except Exception:
                     pass
 
         return outputs
 
-    def wait_for_completion(self, other_pipes: List["Pipeline[Any]"] = []) -> None:
-        return Pipeline.wait_for_all_completions([self, *other_pipes])
+    def wait_for_completion(
+        self, other_pipes: List["Pipeline[Any]"] = [], debug_graph: Optional[str] = None
+    ) -> None:
+        """_summary_
+
+        Args:
+            other_pipes (List[&quot;Pipeline[Any]&quot;], optional): _description_. Defaults to [].
+            debug_graph (Optional[str], optional): path to save the dot graph file. Defaults to None.
+
+        Returns:
+            _type_: _description_
+        """
+        if debug_graph is not None:
+            self.build_dot_graph(debug_graph)
+        return Pipeline._wait_for_all_completions([self, *other_pipes])
 
     def wait_for_results(
-        self, other_pipes: List["Pipeline[Any]"] = []
+        self, other_pipes: List["Pipeline[Any]"] = [], debug_graph: Optional[str] = None
     ) -> List[List[R]]:
-        return Pipeline.wait_for_all_results([self, *other_pipes])
+        """_summary_
 
-    def wait_for_result(self) -> List[R]:
-        [res] = Pipeline.wait_for_all_results([self])
+        Args:
+            other_pipes (List[&quot;Pipeline[Any]&quot;], optional): _description_. Defaults to [].
+            debug_graph (Optional[str], optional): path to save the dot graph file. Defaults to None.
+
+        Returns:
+            List[List[R]]: _description_
+        """
+        if debug_graph is not None:
+            self.build_dot_graph(debug_graph)
+        return Pipeline._wait_for_all_results([self, *other_pipes])
+
+    def wait_for_result(self, debug_graph: Optional[str] = None) -> List[R]:
+        """
+        Args:
+            debug_graph (Optional[str], optional): path to save the dot graph file. Defaults to None.
+
+        Returns:
+            List[R]: _description_
+        """
+        if debug_graph is not None:
+            self.build_dot_graph(debug_graph)
+        [res] = Pipeline._wait_for_all_results([self])
         return res
